@@ -4,12 +4,14 @@ use std::io::{Cursor, Seek, SeekFrom, Write};
 
 use crate::{
     anim::*,
-    formats::{mesh::*, nrpd::RenderPassDataType},
+    formats::{
+        mesh::*,
+        nrpd::{FrameBuffer, NrpdState, RenderPassDataType},
+    },
     matl::*,
     shdr::*,
     skel::*,
-    InlineString, RelPtr64, SsbhArray, SsbhByteBuffer, SsbhFile, SsbhString, SsbhString8,
-    SsbhWrite,
+    CString, RelPtr64, SsbhArray, SsbhByteBuffer, SsbhFile, SsbhWrite,
 };
 
 fn round_up(value: u64, n: u64) -> u64 {
@@ -95,6 +97,124 @@ macro_rules! ssbh_write_impl {
 
 ssbh_write_impl!(u8, u16, u32, u64, u128, i8, i16, i32, i64, i128, f32, f64);
 
+impl SsbhWrite for VertexWeights {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        match self {
+            VertexWeights::VertexWeightsV8(v) => v.write_ssbh(writer, data_ptr),
+            VertexWeights::VertexWeightsV10(v) => v.write_ssbh(writer, data_ptr),
+        }
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        match self {
+            VertexWeights::VertexWeightsV8(v) => v.size_in_bytes(),
+            VertexWeights::VertexWeightsV10(v) => v.size_in_bytes(),
+        }
+    }
+}
+
+impl SsbhWrite for MeshAttributes {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        match self {
+            MeshAttributes::AttributesV8(v) => v.write_ssbh(writer, data_ptr),
+            MeshAttributes::AttributesV10(v) => v.write_ssbh(writer, data_ptr),
+        }
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        match self {
+            MeshAttributes::AttributesV8(v) => v.size_in_bytes(),
+            MeshAttributes::AttributesV10(v) => v.size_in_bytes(),
+        }
+    }
+}
+
+impl SsbhWrite for NrpdState {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        match self {
+            NrpdState::Sampler(v) => v.write_ssbh(writer, data_ptr),
+            NrpdState::RasterizerState(v) => v.write_ssbh(writer, data_ptr),
+            NrpdState::DepthState(v) => v.write_ssbh(writer, data_ptr),
+            NrpdState::BlendState(v) => v.write_ssbh(writer, data_ptr),
+        }
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        match self {
+            NrpdState::Sampler(v) => v.size_in_bytes(),
+            NrpdState::RasterizerState(v) => v.size_in_bytes(),
+            NrpdState::DepthState(v) => v.size_in_bytes(),
+            NrpdState::BlendState(v) => v.size_in_bytes(),
+        }
+    }
+}
+
+impl SsbhWrite for FrameBuffer {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        match self {
+            FrameBuffer::Framebuffer0(v) => v.write_ssbh(writer, data_ptr),
+            FrameBuffer::Framebuffer1(v) => v.write_ssbh(writer, data_ptr),
+            FrameBuffer::Framebuffer2(v) => v.write_ssbh(writer, data_ptr),
+        }
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        match self {
+            FrameBuffer::Framebuffer0(v) => v.size_in_bytes(),
+            FrameBuffer::Framebuffer1(v) => v.size_in_bytes(),
+            FrameBuffer::Framebuffer2(v) => v.size_in_bytes(),
+        }
+    }
+}
+
+impl SsbhWrite for Param {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
+        &self,
+        writer: &mut W,
+        data_ptr: &mut u64,
+    ) -> std::io::Result<()> {
+        match self {
+            Param::Float(v) => v.write_ssbh(writer, data_ptr),
+            Param::Boolean(v) => v.write_ssbh(writer, data_ptr),
+            Param::Vector4(v) => v.write_ssbh(writer, data_ptr),
+            Param::MatlString(v) => v.write_ssbh(writer, data_ptr),
+            Param::Sampler(v) => v.write_ssbh(writer, data_ptr),
+            Param::UvTransform(v) => v.write_ssbh(writer, data_ptr),
+            Param::BlendState(v) => v.write_ssbh(writer, data_ptr),
+            Param::RasterizerState(v) => v.write_ssbh(writer, data_ptr),
+        }
+    }
+
+    fn size_in_bytes(&self) -> u64 {
+        match self {
+            Param::Float(v) => v.size_in_bytes(),
+            Param::Boolean(v) => v.size_in_bytes(),
+            Param::Vector4(v) => v.size_in_bytes(),
+            Param::MatlString(v) => v.size_in_bytes(),
+            Param::Sampler(v) => v.size_in_bytes(),
+            Param::UvTransform(v) => v.size_in_bytes(),
+            Param::BlendState(v) => v.size_in_bytes(),
+            Param::RasterizerState(v) => v.size_in_bytes(),
+        }
+    }
+}
+
 impl<T: binread::BinRead + SsbhWrite> SsbhWrite for Option<T> {
     fn write_ssbh<W: Write + Seek>(
         &self,
@@ -324,128 +444,29 @@ fn write_ssbh_header<W: Write + Seek>(writer: &mut W, magic: &[u8; 4]) -> std::i
     Ok(())
 }
 
-// TODO: This could be derived.
-impl SsbhWrite for SsbhString {
-    fn write_ssbh<W: Write + Seek>(
+impl<const N: usize> SsbhWrite for CString<N> {
+    fn write_ssbh<W: std::io::Write + std::io::Seek>(
         &self,
         writer: &mut W,
-        data_ptr: &mut u64,
+        _data_ptr: &mut u64,
     ) -> std::io::Result<()> {
-        // The data pointer must point past the containing struct.
-        let current_pos = writer.stream_position()?;
-        if *data_ptr < current_pos + self.size_in_bytes() {
-            *data_ptr = current_pos + self.size_in_bytes();
-        }
-
-        // TODO: This is shared with ssbh string8 but has 4 byte alignment and 4 byte empty strings.
-        match &self.0 .0 {
-            Some(value) => {
-                // Calculate the relative offset.
-                *data_ptr = round_up(*data_ptr, 4);
-                write_relative_offset(writer, data_ptr)?;
-
-                // Write the data at the specified offset.
-                let pos_after_offset = writer.stream_position()?;
-                writer.seek(SeekFrom::Start(*data_ptr))?;
-
-                // TODO: Find a nicer way to handle this.
-                if value.0.is_empty() {
-                    // 4 byte empty strings.
-                    writer.write_all(&[0u8; 4])?;
-                } else {
-                    value.write_ssbh(writer, data_ptr)?;
-                }
-
-                // Point the data pointer past the current write.
-                // Types with relative offsets will already increment the data pointer.
-                let current_pos = writer.stream_position()?;
-                if current_pos > *data_ptr {
-                    *data_ptr = round_up(current_pos, 4);
-                }
-
-                writer.seek(SeekFrom::Start(pos_after_offset))?;
-                Ok(())
-            }
-            None => {
-                // Null offsets don't increment the data pointer.
-                writer.write_u64::<LittleEndian>(0u64)?;
-                Ok(())
-            }
+        if self.0 .0.len() == 0 {
+            // Handle empty strings.
+            writer.write_all(&[0u8; N])?;
+        } else {
+            // Write the data and null terminator.
+            writer.write_all(&self.0 .0)?;
+            writer.write_all(&[0u8])?;
         }
+        Ok(())
     }
 
     fn size_in_bytes(&self) -> u64 {
         self.0.size_in_bytes()
     }
-}
 
-// TODO: This could be derived.
-impl SsbhWrite for InlineString {
-    fn write_ssbh<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        self.0.write_ssbh(writer, data_ptr)
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        self.0.size_in_bytes()
-    }
-}
-
-// TODO: This could just be derived as RelPtr64<NullString> but requires different alignment.
-impl SsbhWrite for SsbhString8 {
-    fn write_ssbh<W: Write + Seek>(
-        &self,
-        writer: &mut W,
-        data_ptr: &mut u64,
-    ) -> std::io::Result<()> {
-        // The data pointer must point past the containing struct.
-        let current_pos = writer.stream_position()?;
-        if *data_ptr < current_pos + self.size_in_bytes() {
-            *data_ptr = current_pos + self.size_in_bytes();
-        }
-
-        // TODO: This is shared with ssbh string but has 8 byte alignment and 8 byte empty strings.
-        match &self.0 .0 .0 {
-            Some(value) => {
-                // Calculate the relative offset.
-                *data_ptr = round_up(*data_ptr, 8);
-                write_relative_offset(writer, data_ptr)?;
-
-                // Write the data at the specified offset.
-                let pos_after_offset = writer.stream_position()?;
-                writer.seek(SeekFrom::Start(*data_ptr))?;
-
-                // TODO: Find a nicer way to handle this.
-                if value.0.is_empty() {
-                    //8 byte empty strings.
-                    writer.write_all(&[0u8; 8])?;
-                } else {
-                    value.write_ssbh(writer, data_ptr)?;
-                }
-
-                // Point the data pointer past the current write.
-                // Types with relative offsets will already increment the data pointer.
-                let current_pos = writer.stream_position()?;
-                if current_pos > *data_ptr {
-                    *data_ptr = round_up(current_pos, 8);
-                }
-
-                writer.seek(SeekFrom::Start(pos_after_offset))?;
-                Ok(())
-            }
-            None => {
-                // Null offsets don't increment the data pointer.
-                writer.write_u64::<LittleEndian>(0u64)?;
-                Ok(())
-            }
-        }
-    }
-
-    fn size_in_bytes(&self) -> u64 {
-        self.0.size_in_bytes()
+    fn alignment_in_bytes(&self) -> u64 {
+        N as u64
     }
 }
 
@@ -564,12 +585,16 @@ pub fn write_ssbh_file<W: Write + Seek, S: SsbhWrite>(
 #[cfg(test)]
 mod tests {
     // The tests are designed to check the SSBH offset rules.
-    // 1. Offsets point past the containing struct.
-    // 2. Offsets in array elements point past the containing array.
-    // 3. Offsets obey the alignment rules of the data's type.
+    // It's unclear if these rules are strictly required by the format or in game parsers,
+    // but following these rules creates 1:1 export for all formats except NRPD.
+
+    // 1. Offsets are nonnegative.
+    // 2. Offsets point past the containing struct.
+    // 3. Offsets in array elements point past the containing array.
+    // 4. Offsets obey the alignment rules of the pointed to data's type.
 
     use super::*;
-    use crate::{SsbhEnum64, SsbhString};
+    use crate::{SsbhEnum64, SsbhString, SsbhString8};
     use binread::BinRead;
 
     fn hex_bytes(hex: &str) -> Vec<u8> {
